@@ -44,11 +44,13 @@ import (
 
 // grepCmd represents the grep command
 var grepCmd = &cobra.Command{
+	GroupID: "search",
+
 	Use:   "grep",
 	Short: "search sequences by ID/name/sequence/sequence motifs, mismatch allowed",
 	Long: fmt.Sprintf(`search sequences by ID/name/sequence/sequence motifs, mismatch allowed
 
-Attentions:
+Attention:
 
   0. By default, we match sequence ID with patterns, use "-n/--by-name"
      for matching full name instead of just ID.
@@ -57,6 +59,8 @@ Attentions:
      for partly matching.
   2. When searching by sequences, it's partly matching, and both positive
      and negative strands are searched.
+     Please switch on "-P/--only-positive-strand" if you would like to
+     search only on the positive strand.
      Mismatch is allowed using flag "-m/--max-mismatch", you can increase
      the value of "-j/--threads" to accelerate processing.
   3. Degenerate bases/residues like "RYMM.." are also supported by flag -d.
@@ -110,6 +114,7 @@ Examples:
 		degenerate := getFlagBool(cmd, "degenerate")
 		region := getFlagString(cmd, "region")
 		circular := getFlagBool(cmd, "circular")
+		allowDups := getFlagBool(cmd, "allow-duplicated-patterns")
 
 		immediateOutput := getFlagBool(cmd, "immediate-output")
 
@@ -184,7 +189,7 @@ Examples:
 
 		// prepare pattern
 		patternsR := make(map[uint64]*regexp.Regexp, 1<<10)
-		patternsN := make(map[uint64]interface{}, 1<<20)
+		patternsN := make(map[uint64]int, 1<<20)
 		// patternsS := make(map[string]interface{}, 1<<10)
 		patternsS := make([][]byte, 0, 16)
 
@@ -246,9 +251,9 @@ Examples:
 						}
 					} else {
 						if ignoreCase {
-							patternsN[xxhash.Sum64String(strings.ToLower(p))] = struct{}{}
+							patternsN[xxhash.Sum64String(strings.ToLower(p))]++
 						} else {
-							patternsN[xxhash.Sum64String(p)] = struct{}{}
+							patternsN[xxhash.Sum64String(p)]++
 						}
 					}
 				}
@@ -306,9 +311,9 @@ Examples:
 					}
 				} else {
 					if ignoreCase {
-						patternsN[xxhash.Sum64String(strings.ToLower(p))] = struct{}{}
+						patternsN[xxhash.Sum64String(strings.ToLower(p))]++
 					} else {
-						patternsN[xxhash.Sum64String(p)] = struct{}{}
+						patternsN[xxhash.Sum64String(p)]++
 					}
 				}
 			}
@@ -318,7 +323,6 @@ Examples:
 		checkError(err)
 		defer outfh.Close()
 
-		var fastxReader *fastx.Reader
 		var record *fastx.Record
 		strands := []byte{'+', '-'}
 
@@ -348,7 +352,9 @@ Examples:
 				id = 1
 				for r := range ch {
 					if justCount {
-						count++
+						if r.ok {
+							count++
+						}
 						continue
 					}
 
@@ -403,7 +409,7 @@ Examples:
 
 			var id uint64
 			for _, file := range files {
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 
 				checkAlphabet := true
@@ -512,6 +518,7 @@ Examples:
 
 					}(record.Clone(), id)
 				}
+				fastxReader.Close()
 			}
 
 			wg.Wait()
@@ -534,8 +541,9 @@ Examples:
 		var re *regexp.Regexp
 		var h uint64
 		var strand byte
+		var i, n int // for output records multiple times when duplicated patterns are given.
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			checkAlphabet := true
@@ -575,6 +583,8 @@ Examples:
 
 				hit = false
 
+				n = 1
+
 				for _, strand = range strands {
 					if hit {
 						break
@@ -655,7 +665,7 @@ Examples:
 						if ignoreCase {
 							h = xxhash.Sum64(bytes.ToLower(target))
 						}
-						if _, ok = patternsN[h]; ok {
+						if n, ok = patternsN[h]; ok {
 							hit = true
 							if deleteMatched && !invertMatch {
 								delete(patternsN, h)
@@ -677,14 +687,23 @@ Examples:
 
 				if justCount {
 					count++
+					if allowDups && n > 1 {
+						count += n - 1
+					}
 				} else {
 					record.FormatToWriter(outfh, config.LineWidth)
+					if allowDups && n > 1 {
+						for i = 0; i < n-1; i++ {
+							record.FormatToWriter(outfh, config.LineWidth)
+						}
+					}
 				}
 
 				if immediateOutput {
 					outfh.Flush()
 				}
 			}
+			fastxReader.Close()
 
 			config.LineWidth = lineWidth
 		}
@@ -698,14 +717,15 @@ Examples:
 func init() {
 	RootCmd.AddCommand(grepCmd)
 
-	grepCmd.Flags().StringSliceP("pattern", "p", []string{""}, `search pattern (multiple values supported. Attention: use double quotation marks for patterns containing comma, e.g., -p '"A{2,}"'))`)
+	grepCmd.Flags().StringSliceP("pattern", "p", []string{""}, `search pattern (multiple values supported. Attention: use double quotation marks for patterns containing comma, e.g., -p '"A{2,}"')`)
+	grepCmd.Flags().BoolP("allow-duplicated-patterns", "D", false, "output records multiple times when duplicated patterns are given")
 	grepCmd.Flags().StringP("pattern-file", "f", "", "pattern file (one record per line)")
 	grepCmd.Flags().BoolP("use-regexp", "r", false, "patterns are regular expression")
 	grepCmd.Flags().BoolP("delete-matched", "", false, "delete a pattern right after being matched, this keeps the firstly matched data and speedups when using regular expressions")
 	grepCmd.Flags().BoolP("invert-match", "v", false, "invert the sense of matching, to select non-matching records")
 	grepCmd.Flags().BoolP("by-name", "n", false, "match by full name instead of just ID")
-	grepCmd.Flags().BoolP("by-seq", "s", false, "search subseq on seq, both positive and negative strand are searched, and mismatch allowed using flag -m/--max-mismatch")
-	grepCmd.Flags().BoolP("only-positive-strand", "P", false, "only search on positive strand")
+	grepCmd.Flags().BoolP("by-seq", "s", false, "search subseq on seq. Both positive and negative strand are searched by default, you might use -P/--only-positive-strand. Mismatch allowed using flag -m/--max-mismatch")
+	grepCmd.Flags().BoolP("only-positive-strand", "P", false, "only search on the positive strand")
 	grepCmd.Flags().IntP("max-mismatch", "m", 0, "max mismatch when matching by seq. For large genomes like human genome, using mapping/alignment tools would be faster")
 	grepCmd.Flags().BoolP("ignore-case", "i", false, "ignore case")
 	grepCmd.Flags().BoolP("degenerate", "d", false, "pattern/motif contains degenerate base")
