@@ -69,9 +69,15 @@ func getFileList(args []string, checkFile bool) []string {
 }
 
 func getFileListFromFile(file string, checkFile bool) ([]string, error) {
-	fh, err := os.Open(file)
-	if err != nil {
-		return nil, fmt.Errorf("read file list from '%s': %s", file, err)
+	var fh *os.File
+	var err error
+	if file == "-" {
+		fh = os.Stdin
+	} else {
+		fh, err = os.Open(file)
+		if err != nil {
+			return nil, fmt.Errorf("read file list from '%s': %s", file, err)
+		}
 	}
 
 	var _file string
@@ -169,6 +175,12 @@ func getFlagStringSlice(cmd *cobra.Command, flag string) []string {
 	return value
 }
 
+func getFlagFloat64Slice(cmd *cobra.Command, flag string) []float64 {
+	value, err := cmd.Flags().GetFloat64Slice(flag)
+	checkError(err)
+	return value
+}
+
 func getIDRegexp(cmd *cobra.Command, flag string) string {
 	var idRegexp string
 	f := getFlagBool(cmd, "id-ncbi")
@@ -232,6 +244,7 @@ type Config struct {
 	Quiet                  bool
 	AlphabetGuessSeqLength int
 	ValidateSeqLength      int
+	CompressionLevel       int
 }
 
 func getConfigs(cmd *cobra.Command) Config {
@@ -239,6 +252,37 @@ func getConfigs(cmd *cobra.Command) Config {
 	if threads >= 1000 {
 		checkError(fmt.Errorf("are your seriously? %d threads? It will exhaust your RAM", threads))
 	}
+	outfile := getFlagString(cmd, "out-file")
+
+	level := getFlagInt(cmd, "compress-level")
+
+	f := strings.ToLower(outfile)
+	var ranges [3]int
+	var format string
+	checkCompressLevel := true
+	if strings.HasSuffix(f, ".gz") {
+		format = "gzip"
+		ranges = [3]int{1, 9, 5}
+	} else if strings.HasSuffix(f, ".xz") {
+		checkCompressLevel = false
+	} else if strings.HasSuffix(f, ".zst") {
+		format = "zstd"
+		ranges = [3]int{1, 4, 2}
+	} else if strings.HasSuffix(f, ".bz2") {
+		format = "bzip2"
+		ranges = [3]int{1, 9, 6}
+	} else {
+		checkCompressLevel = false
+	}
+	if checkCompressLevel {
+		if level == -1 {
+			level = ranges[2]
+		}
+		if level < ranges[0] || level > ranges[1] {
+			checkError(fmt.Errorf("compression level out of range for %s format (%d-%d): %d", format, ranges[0], ranges[1], level))
+		}
+	}
+	xopen.Level = level
 
 	return Config{
 		Alphabet:               getAlphabet(cmd, "seq-type"),
@@ -246,9 +290,10 @@ func getConfigs(cmd *cobra.Command) Config {
 		LineWidth:              getFlagNonNegativeInt(cmd, "line-width"),
 		IDRegexp:               getIDRegexp(cmd, "id-regexp"),
 		IDNCBI:                 getFlagBool(cmd, "id-ncbi"),
-		OutFile:                getFlagString(cmd, "out-file"),
+		OutFile:                outfile,
 		Quiet:                  getFlagBool(cmd, "quiet"),
 		AlphabetGuessSeqLength: getFlagAlphabetGuessSeqLength(cmd, "alphabet-guess-seq-length"),
+		CompressionLevel:       level,
 	}
 
 }
@@ -265,18 +310,56 @@ func sortRecordChunkMapID(chunks map[uint64]fastx.RecordChunk) sortutil.Uint64Sl
 }
 
 func filepathTrimExtension(file string) (string, string) {
-	gz := strings.HasSuffix(file, ".gz") || strings.HasSuffix(file, ".GZ")
-	if gz {
+	var _gz, _xz, _zst bool
+	_gz = strings.HasSuffix(file, ".gz") || strings.HasSuffix(file, ".GZ")
+	if _gz {
 		file = file[0 : len(file)-3]
+	} else {
+		_xz = strings.HasSuffix(file, ".xz") || strings.HasSuffix(file, ".XZ")
+		if _xz {
+			file = file[0 : len(file)-3]
+		} else {
+			_zst = strings.HasSuffix(file, ".zst") || strings.HasSuffix(file, ".ZST")
+			if _zst {
+				file = file[0 : len(file)-4]
+			}
+		}
 	}
+
 	extension := filepath.Ext(file)
 	name := file[0 : len(file)-len(extension)]
-	if gz {
+	if _gz {
 		extension += ".gz"
+	} else if _xz {
+		extension += ".xz"
+	} else if _zst {
+		extension += ".zst"
 	}
 	return name, extension
 }
 
+func filepathTrimExtension2(file string, suffixes []string) (string, string, string) {
+	if suffixes == nil {
+		suffixes = []string{".gz", ".xz", ".zst"}
+	}
+
+	var e, e1, e2 string
+	f := strings.ToLower(file)
+	for _, s := range suffixes {
+		e = strings.ToLower(s)
+		if strings.HasSuffix(f, e) {
+			e2 = e
+			file = file[0 : len(file)-len(e)]
+			break
+		}
+	}
+
+	e1 = filepath.Ext(file)
+	name := file[0 : len(file)-len(e1)]
+
+	return name, e1, e2
+}
+
 func isPlainFile(file string) bool {
 	return file != "" && !strings.HasSuffix(strings.ToLower(file), ".gz")
 }
@@ -318,15 +401,20 @@ func copySeqs(file, newFile string) (int, error) {
 	return n, nil
 }
 
-func getFaidx(file string, idRegexp string) *fai.Faidx {
+func getFaidx(file string, idRegexp string, quiet bool) *fai.Faidx {
 	var idx fai.Index
 	var err error
 	fileFai := file + ".seqkit.fai"
 	if fileNotExists(fileFai) {
-		log.Infof("create FASTA index for %s", file)
+		if !quiet {
+			log.Infof("create FASTA index for %s", file)
+		}
 		idx, err = fai.CreateWithIDRegexp(file, fileFai, idRegexp)
 		checkError(err)
 	} else {
+		if !quiet {
+			log.Infof("read FASTA index from %s", fileFai)
+		}
 		idx, err = fai.Read(fileFai)
 		checkError(err)
 	}
