@@ -39,11 +39,13 @@ import (
 
 // locateCmd represents the locate command
 var locateCmd = &cobra.Command{
+	GroupID: "search",
+
 	Use:   "locate",
 	Short: "locate subsequences/motifs, mismatch allowed",
 	Long: `locate subsequences/motifs, mismatch allowed
 
-Attentions:
+Attention:
 
   1. Motifs could be EITHER plain sequence containing "ACTGN" OR regular
      expression like "A[TU]G(?:.{3})+?[TU](?:AG|AA|GA)" for ORFs.     
@@ -68,8 +70,6 @@ Attentions:
 		outFile := config.OutFile
 		seq.AlphabetGuessSeqLengthThreshold = config.AlphabetGuessSeqLength
 		seq.ValidateSeq = true
-		seq.ValidateWholeSeq = false
-		seq.ValidSeqLengthThreshold = getFlagValidateSeqLength(cmd, "validate-seq-length")
 		seq.ValidSeqThreads = config.Threads
 		seq.ComplementThreads = config.Threads
 		quiet := config.Quiet
@@ -92,6 +92,7 @@ Attentions:
 		mismatches := getFlagNonNegativeInt(cmd, "max-mismatch")
 		hideMatched := getFlagBool(cmd, "hide-matched")
 		circular := getFlagBool(cmd, "circular")
+		len2show := getFlagNonNegativeInt(cmd, "max-len-to-show")
 
 		immediateOutput := getFlagBool(cmd, "immediate-output")
 
@@ -124,7 +125,7 @@ Attentions:
 				checkError(fmt.Errorf("flag -d (--degenerate) not allowed when giving flag -m (--max-mismatch)"))
 			}
 			if useRegexp {
-				checkError(fmt.Errorf("flag -r (--use-regexp) not allowed when giving flag -m (--use-regexp)"))
+				checkError(fmt.Errorf("flag -r (--use-regexp) not allowed when giving flag -m (--max-mismatch)"))
 			}
 			if nonGreedy && !quiet {
 				log.Infof("flag -G (--non-greedy) ignored when giving flag -m (--max-mismatch)")
@@ -260,7 +261,6 @@ Attentions:
 		// only for m > 0, where FMI is slow
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		_onlyPositiveStrand := onlyPositiveStrand
 
 		if mismatches > 0 || useFMI {
@@ -344,7 +344,7 @@ Attentions:
 
 			var id uint64
 			for _, file := range files {
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 
 				checkAlphabet := true
@@ -456,7 +456,7 @@ Attentions:
 											_ch <- fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 												record.ID,
 												pName,
-												patterns[pName],
+												prune(patterns[pName], len2show), // patterns[pName],
 												"+",
 												begin,
 												end)
@@ -464,11 +464,11 @@ Attentions:
 											_ch <- fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
 												record.ID,
 												pName,
-												patterns[pName],
+												prune(patterns[pName], len2show), // patterns[pName],
 												"+",
 												begin,
 												end,
-												record.Seq.Seq[i:i+len(pSeq)])
+												prune(record.Seq.Seq[i:i+len(pSeq)], len2show)) // record.Seq.Seq[i:i+len(pSeq)])
 										}
 									}
 								}
@@ -541,7 +541,7 @@ Attentions:
 											_ch <- fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 												record.ID,
 												pName,
-												patterns[pName],
+												prune(patterns[pName], len2show), // patterns[pName],
 												"-",
 												begin,
 												end)
@@ -549,11 +549,11 @@ Attentions:
 											_ch <- fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
 												record.ID,
 												pName,
-												patterns[pName],
+												prune(patterns[pName], len2show), // patterns[pName],
 												"-",
 												begin,
 												end,
-												seqRP.Seq[i:i+len(pSeq)])
+												prune(seqRP.Seq[i:i+len(pSeq)], len2show)) // seqRP.Seq[i:i+len(pSeq)])
 										}
 									}
 								}
@@ -570,6 +570,7 @@ Attentions:
 						ch <- &Arecord{record: results, id: id, ok: len(results) > 0}
 					}(record.Clone(), id)
 				}
+				fastxReader.Close()
 			}
 
 			wg.Wait()
@@ -584,9 +585,11 @@ Attentions:
 		var seqRP *seq.Seq
 		var offset, l, lpatten int
 		var loc []int
-		var locs, locsNeg [][2]int
+		// var locs, locsNeg [][2]int
+		// locs = make([][2]int, 0, 1024)
+		// locsNeg = make([][2]int, 0, 1024)
 		var i, begin, end int
-		var flag bool
+		// var flag bool
 		var pSeq, p []byte
 		var pName string
 		var re *regexp.Regexp
@@ -596,7 +599,7 @@ Attentions:
 		}
 
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			checkAlphabet := true
@@ -677,7 +680,7 @@ Attentions:
 									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 										record.ID,
 										pName,
-										patterns[pName],
+										prune(patterns[pName], len2show), // patterns[pName],
 										"+",
 										begin,
 										end))
@@ -685,11 +688,11 @@ Attentions:
 									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
 										record.ID,
 										pName,
-										patterns[pName],
+										prune(patterns[pName], len2show), // patterns[pName],
 										"+",
 										begin,
 										end,
-										record.Seq.Seq[i:i+len(pSeq)]))
+										prune(record.Seq.Seq[i:i+len(pSeq)], len2show))) // record.Seq.Seq[i:i+len(pSeq)]))
 								}
 							}
 						}
@@ -744,7 +747,7 @@ Attentions:
 									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 										record.ID,
 										pName,
-										patterns[pName],
+										prune(patterns[pName], len2show), // patterns[pName],
 										"-",
 										begin,
 										end))
@@ -752,11 +755,11 @@ Attentions:
 									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
 										record.ID,
 										pName,
-										patterns[pName],
+										prune(patterns[pName], len2show), // patterns[pName],
 										"-",
 										begin,
 										end,
-										seqRP.Seq[i:i+len(pSeq)]))
+										prune(seqRP.Seq[i:i+len(pSeq)], len2show))) // seqRP.Seq[i:i+len(pSeq)]))
 								}
 							}
 						}
@@ -770,7 +773,7 @@ Attentions:
 				}
 
 				for pName = range patterns {
-					locs = make([][2]int, 0, 1000)
+					// locs = locs[:0]
 
 					offset = 0
 					if !(useRegexp || degenerate) {
@@ -800,58 +803,58 @@ Attentions:
 
 						end = offset + loc[1]
 
-						flag = true // check "duplicated" region
-						if useRegexp || degenerate {
-							for i = len(locs) - 1; i >= 0; i-- {
-								if locs[i][0] <= begin && locs[i][1] >= end {
-									flag = false
-									break
-								}
-							}
-						}
+						// flag = true // check "embedded" region
+						// if useRegexp || degenerate {
+						// 	for i = len(locs) - 1; i >= 0; i-- {
+						// 		if locs[i][0] <= begin && locs[i][1] >= end {
+						// 			flag = false
+						// 			break
+						// 		}
+						// 	}
+						// }
 
-						if flag {
-							if outFmtGTF {
-								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\tgene_id \"%s\"; \n",
+						// if flag {
+						if outFmtGTF {
+							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\tgene_id \"%s\"; \n",
+								record.ID,
+								"SeqKit",
+								"location",
+								begin,
+								end,
+								0,
+								"+",
+								".",
+								pName))
+						} else if outFmtBED {
+							outfh.WriteString(fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\n",
+								record.ID,
+								begin-1,
+								end,
+								pName,
+								0,
+								"+"))
+						} else {
+							if hideMatched {
+								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 									record.ID,
-									"SeqKit",
-									"location",
+									pName,
+									prune(patterns[pName], len2show), // patterns[pName],
+									"+",
+									begin,
+									end))
+							} else {
+								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
+									record.ID,
+									pName,
+									prune(patterns[pName], len2show), // patterns[pName],
+									"+",
 									begin,
 									end,
-									0,
-									"+",
-									".",
-									pName))
-							} else if outFmtBED {
-								outfh.WriteString(fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\n",
-									record.ID,
-									begin-1,
-									end,
-									pName,
-									0,
-									"+"))
-							} else {
-								if hideMatched {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
-										record.ID,
-										pName,
-										patterns[pName],
-										"+",
-										begin,
-										end))
-								} else {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
-										record.ID,
-										pName,
-										patterns[pName],
-										"+",
-										begin,
-										end,
-										record.Seq.Seq[begin-1:end]))
-								}
+									prune(record.Seq.Seq[begin-1:end], len2show))) // record.Seq.Seq[begin-1:end]))
 							}
-							locs = append(locs, [2]int{begin, end})
 						}
+						// locs = append(locs, [2]int{begin, end})
+						// }
 
 						if nonGreedy {
 							offset = offset + loc[1] + 1
@@ -869,7 +872,7 @@ Attentions:
 
 					seqRP = record.Seq.RevCom()
 
-					locsNeg = make([][2]int, 0, 1000)
+					// locsNeg = locsNeg[:0]
 
 					offset = 0
 
@@ -899,58 +902,58 @@ Attentions:
 							end += l
 						}
 
-						flag = true
-						if useRegexp || degenerate {
-							for i = len(locsNeg) - 1; i >= 0; i-- {
-								if locsNeg[i][0] <= begin && locsNeg[i][1] >= end {
-									flag = false
-									break
-								}
-							}
-						}
+						// flag = true
+						// if useRegexp || degenerate {
+						// 	for i = len(locsNeg) - 1; i >= 0; i-- {
+						// 		if locsNeg[i][0] <= begin && locsNeg[i][1] >= end {
+						// 			flag = false
+						// 			break
+						// 		}
+						// 	}
+						// }
 
-						if flag {
-							if outFmtGTF {
-								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\tgene_id \"%s\"; \n",
+						// if flag {
+						if outFmtGTF {
+							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%s\t%s\tgene_id \"%s\"; \n",
+								record.ID,
+								"SeqKit",
+								"location",
+								begin,
+								end,
+								0,
+								"-",
+								".",
+								pName))
+						} else if outFmtBED {
+							outfh.WriteString(fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\n",
+								record.ID,
+								begin-1,
+								end,
+								pName,
+								0,
+								"-"))
+						} else {
+							if hideMatched {
+								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
 									record.ID,
-									"SeqKit",
-									"location",
+									pName,
+									prune(patterns[pName], len2show), // patterns[pName],
+									"-",
+									begin,
+									end))
+							} else {
+								outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
+									record.ID,
+									pName,
+									prune(patterns[pName], len2show), // patterns[pName],
+									"-",
 									begin,
 									end,
-									0,
-									"-",
-									".",
-									pName))
-							} else if outFmtBED {
-								outfh.WriteString(fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\n",
-									record.ID,
-									begin-1,
-									end,
-									pName,
-									0,
-									"-"))
-							} else {
-								if hideMatched {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\n",
-										record.ID,
-										pName,
-										patterns[pName],
-										"-",
-										begin,
-										end))
-								} else {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%s\t%d\t%d\t%s\n",
-										record.ID,
-										pName,
-										patterns[pName],
-										"-",
-										begin,
-										end,
-										seqRP.Seq[offset+loc[0]:offset+loc[1]]))
-								}
+									prune(seqRP.Seq[offset+loc[0]:offset+loc[1]], len2show))) // seqRP.Seq[offset+loc[0]:offset+loc[1]]))
 							}
-							locsNeg = append(locsNeg, [2]int{begin, end})
 						}
+						// locsNeg = append(locsNeg, [2]int{begin, end})
+						// }
 
 						if nonGreedy {
 							offset = offset + loc[1] + 1
@@ -967,6 +970,7 @@ Attentions:
 					outfh.Flush()
 				}
 			}
+			fastxReader.Close()
 		}
 
 	},
@@ -982,12 +986,25 @@ func init() {
 	locateCmd.Flags().BoolP("use-fmi", "F", false, "use FM-index for much faster search of lots of sequence patterns")
 	locateCmd.Flags().BoolP("ignore-case", "i", false, "ignore case")
 	locateCmd.Flags().BoolP("only-positive-strand", "P", false, "only search on positive strand")
-	locateCmd.Flags().IntP("validate-seq-length", "V", 10000, "length of sequence to validate (0 for whole seq)")
 	locateCmd.Flags().BoolP("non-greedy", "G", false, "non-greedy mode, faster but may miss motifs overlapping with others")
 	locateCmd.Flags().BoolP("gtf", "", false, "output in GTF format")
 	locateCmd.Flags().BoolP("bed", "", false, "output in BED6 format")
 	locateCmd.Flags().IntP("max-mismatch", "m", 0, "max mismatch when matching by seq. For large genomes like human genome, using mapping/alignment tools would be faster")
 	locateCmd.Flags().BoolP("hide-matched", "M", false, "do not show matched sequences")
+	locateCmd.Flags().IntP("max-len-to-show", "s", 0, "show at most X characters for the search pattern or matched sequences")
 	locateCmd.Flags().BoolP("circular", "c", false, `circular genome. type "seqkit locate -h" for details`)
 	locateCmd.Flags().BoolP("immediate-output", "I", false, "print output immediately, do not use write buffer")
 }
+
+func prune(s []byte, n int) []byte {
+	if n == 0 {
+		return s
+	}
+
+	n0 := len(s)
+	if n0 < n {
+		return s
+	}
+
+	return []byte(string(s[:n]) + "...")
+}
