@@ -23,34 +23,70 @@ package cmd
 import (
 	"fmt"
 	"io"
+	"math"
+	"os"
 	"path/filepath"
 	"runtime"
 	"sort"
+	"strconv"
 	"strings"
 	"sync"
+	"time"
 
 	"github.com/cznic/sortutil"
 	"github.com/dustin/go-humanize"
 	"github.com/shenwei356/bio/seq"
 	"github.com/shenwei356/bio/seqio/fastx"
 	"github.com/shenwei356/bio/util"
+	"github.com/shenwei356/stable"
 	"github.com/shenwei356/util/byteutil"
-	"github.com/shenwei356/util/math"
+	mathutil "github.com/shenwei356/util/math"
 	"github.com/shenwei356/xopen"
 	"github.com/spf13/cobra"
-	"github.com/tatsushid/go-prettytable"
+	"github.com/vbauerster/mpb/v5"
+	"github.com/vbauerster/mpb/v5/decor"
 )
 
 // statCmd represents the stat command
 var statCmd = &cobra.Command{
+	GroupID: "basic",
+
 	Use:     "stats",
 	Aliases: []string{"stat"},
 	Short:   "simple statistics of FASTA/Q files",
 	Long: `simple statistics of FASTA/Q files
 
+Columns:
+
+  1.  file      input file, "-" for STDIN
+  2.  format    FASTA or FASTQ
+  3.  type      DNA, RNA, Protein or Unlimit
+  4.  num_seqs  number of sequences
+  5.  sum_len   number of bases or residues       , with gaps or spaces counted
+  6.  min_len   minimal sequence length           , with gaps or spaces counted
+  7.  avg_len   average sequence length           , with gaps or spaces counted
+  8.  max_len   miximal sequence length           , with gaps or spaces counted
+  9.  Q1        first quartile of sequence length , with gaps or spaces counted
+  10. Q2        median of sequence length         , with gaps or spaces counted
+  11. Q3        third quartile of sequence length , with gaps or spaces counted
+  12. sum_gap   number of gaps
+  13. N50       N50. https://en.wikipedia.org/wiki/N50,_L50,_and_related_statistics#N50
+  14. N50_num   N50_num or L50. https://en.wikipedia.org/wiki/N50,_L50,_and_related_statistics#L50
+  15. Q20(%)    percentage of bases with the quality score greater than 20
+  16. Q30(%)    percentage of bases with the quality score greater than 30
+  17. AvgQual   average quality
+  18. GC(%)     percentage of GC content
+  
+Attention:
+  1. Sequence length metrics (sum_len, min_len, avg_len, max_len, Q1, Q2, Q3)
+     count the number of gaps or spaces. You can remove them with "seqkit seq -g":
+         seqkit seq -g input.fasta | seqkit stats
+
 Tips:
-  1. For lots of small files (especially on SDD), use big value of '-j' to
+  1. For lots of small files (especially on SDD), use a big value of '-j' to
      parallelize counting.
+  2. Extract one metric with csvtk (https://github.com/shenwei356/csvtk):
+         seqkit stats -Ta input.fastq.gz | csvtk cut -t -f "Q30(%)" | csvtk del-header 
 
 `,
 	Run: func(cmd *cobra.Command, args []string) {
@@ -74,6 +110,7 @@ Tips:
 		gapLettersBytes := []byte(gapLetters)
 		gcLettersBytes := []byte{'g', 'c', 'G', 'C'}
 
+		skipFileCheck := getFlagBool(cmd, "skip-file-check")
 		all := getFlagBool(cmd, "all")
 		tabular := getFlagBool(cmd, "tabular")
 		skipErr := getFlagBool(cmd, "skip-err")
@@ -81,8 +118,64 @@ Tips:
 		basename := getFlagBool(cmd, "basename")
 		stdinLabel := getFlagString(cmd, "stdin-label")
 		replaceStdinLabel := stdinLabel != "-"
+		_NX := getFlagStringSlice(cmd, "N")
+		hasNX := len(_NX) > 0
 
-		files := getFileListFromArgsAndFile(cmd, args, true, "infile-list", true)
+		NX := make([]float64, len(_NX))
+		var err error
+		for i, x := range _NX {
+			NX[i], err = strconv.ParseFloat(x, 64)
+			if err != nil {
+				checkError(fmt.Errorf("the value of -N/--N should be a float: %s", x))
+			}
+			if NX[i] < 0 || NX[i] > 100 {
+				checkError(fmt.Errorf("the value of -N/--N should be in the range of [0, 100]: %s", x))
+			}
+		}
+
+		files := getFileListFromArgsAndFile(cmd, args, !skipFileCheck, "infile-list", !skipFileCheck)
+
+		style := &stable.TableStyle{
+			Name: "plain",
+
+			HeaderRow: stable.RowStyle{Begin: "", Sep: "  ", End: ""},
+			DataRow:   stable.RowStyle{Begin: "", Sep: "  ", End: ""},
+			Padding:   "",
+		}
+
+		// process bar
+		var pbs *mpb.Progress
+		var bar *mpb.Bar
+		var chDuration chan time.Duration
+		var doneDuration chan int
+
+		if !config.Quiet && len(files) > 1 {
+			pbs = mpb.New(mpb.WithWidth(40), mpb.WithOutput(os.Stderr))
+			bar = pbs.AddBar(int64(len(files)),
+				mpb.BarStyle("[=>-]<+"),
+				mpb.PrependDecorators(
+					decor.Name("processed files: ", decor.WC{W: len("processed files: "), C: decor.DidentRight}),
+					decor.Name("", decor.WCSyncSpaceR),
+					decor.CountersNoUnit("%d / %d", decor.WCSyncWidth),
+				),
+				mpb.AppendDecorators(
+					decor.Name("ETA: ", decor.WC{W: len("ETA: ")}),
+					decor.EwmaETA(decor.ET_STYLE_GO, 5),
+					decor.OnComplete(decor.Name(""), ". done"),
+				),
+			)
+
+			chDuration = make(chan time.Duration, config.Threads)
+			doneDuration = make(chan int)
+			go func() {
+				for t := range chDuration {
+					bar.Increment()
+					bar.DecoratorEwmaUpdate(t)
+				}
+				doneDuration <- 1
+			}()
+		}
+		// process bar
 
 		outfh, err := xopen.Wopen(outFile)
 		checkError(err)
@@ -101,17 +194,24 @@ Tips:
 				"max_len",
 			}
 			if all {
-				colnames = append(colnames, []string{"Q1", "Q2", "Q3", "sum_gap", "N50", "Q20(%)", "Q30(%)", "GC(%)"}...)
+				colnames = append(colnames, []string{"Q1", "Q2", "Q3", "sum_gap", "N50", "N50_num", "Q20(%)", "Q30(%)", "AvgQual", "GC(%)"}...)
+			}
+
+			if hasNX {
+				for _, x := range _NX {
+					colnames = append(colnames, "N"+x)
+				}
 			}
 			outfh.WriteString(strings.Join(colnames, "\t") + "\n")
 		}
 
 		ch := make(chan statInfo, config.Threads)
-		statInfos := make([]statInfo, 0, 1000)
+		statInfos := make([]statInfo, 0, 1024)
 
 		cancel := make(chan struct{})
 
 		done := make(chan int)
+		var x float64
 		go func() {
 			var id uint64 = 1 // for keepping order
 			buf := make(map[uint64]statInfo)
@@ -132,81 +232,80 @@ Tips:
 					if !tabular {
 						statInfos = append(statInfos, info)
 					} else {
-						if !all {
-							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\n",
-								info.file,
-								info.format,
-								info.t,
-								info.num,
-								info.lenSum,
-								info.lenMin,
-								info.lenAvg,
-								info.lenMax))
-						} else {
-							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\t%.1f\t%.1f\t%.1f\t%d\t%d\t%.2f\t%.2f\t%.2f\n",
-								info.file,
-								info.format,
-								info.t,
-								info.num,
-								info.lenSum,
-								info.lenMin,
-								info.lenAvg,
-								info.lenMax,
+						fmt.Fprintf(outfh, "%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d",
+							info.file,
+							info.format,
+							info.t,
+							info.num,
+							info.lenSum,
+							info.lenMin,
+							info.lenAvg,
+							info.lenMax)
+						if all {
+							fmt.Fprintf(outfh, "\t%.1f\t%.1f\t%.1f\t%d\t%d\t%d\t%.2f\t%.2f\t%.2f\t%.2f",
 								info.Q1,
 								info.Q2,
 								info.Q3,
 								info.gapSum,
 								info.N50,
+								info.L50,
 								info.q20,
 								info.q30,
-								info.gc))
+								info.avgQual,
+								info.gc)
 						}
+						if hasNX {
+							for _, x = range info.nx {
+								fmt.Fprintf(outfh, "\t%.0f", x)
+							}
+						}
+						outfh.WriteString("\n")
+						outfh.Flush()
 					}
 					id++
-				} else { // check bufferd result
-					for true {
-						if info1, ok := buf[id]; ok {
-							if !tabular {
-								statInfos = append(statInfos, info1)
-							} else {
-								if !all {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\n",
-										info1.file,
-										info1.format,
-										info1.t,
-										info1.num,
-										info1.lenSum,
-										info1.lenMin,
-										info1.lenAvg,
-										info1.lenMax))
-								} else {
-									outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\t%.1f\t%.1f\t%.1f\t%d\t%d\t%.2f\t%.2f\t%.2f\n",
-										info1.file,
-										info1.format,
-										info1.t,
-										info1.num,
-										info1.lenSum,
-										info1.lenMin,
-										info1.lenAvg,
-										info1.lenMax,
-										info1.Q1,
-										info1.Q2,
-										info1.Q3,
-										info1.gapSum,
-										info1.N50,
-										info1.q20,
-										info1.q30,
-										info1.gc))
-								}
-							}
+					continue
+				}
 
-							delete(buf, info1.id)
-							id++
-						} else {
-							break
+				buf[info.id] = info // save for later check
+
+				// check bufferd results
+				if info, ok := buf[id]; ok {
+					if !tabular {
+						statInfos = append(statInfos, info)
+					} else {
+						fmt.Fprintf(outfh, "%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d",
+							info.file,
+							info.format,
+							info.t,
+							info.num,
+							info.lenSum,
+							info.lenMin,
+							info.lenAvg,
+							info.lenMax)
+						if all {
+							fmt.Fprintf(outfh, "\t%.1f\t%.1f\t%.1f\t%d\t%d\t%d\t%.2f\t%.2f\t%.2f\t%.2f",
+								info.Q1,
+								info.Q2,
+								info.Q3,
+								info.gapSum,
+								info.N50,
+								info.L50,
+								info.q20,
+								info.q30,
+								info.avgQual,
+								info.gc)
 						}
+						if hasNX {
+							for _, x = range info.nx {
+								fmt.Fprintf(outfh, "\t%.0f", x)
+							}
+						}
+						outfh.WriteString("\n")
+						outfh.Flush()
 					}
-					buf[info.id] = info
+
+					delete(buf, info.id)
+					id++
 				}
 			}
 
@@ -223,35 +322,35 @@ Tips:
 					if !tabular {
 						statInfos = append(statInfos, info)
 					} else {
-						if !all {
-							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\n",
-								info.file,
-								info.format,
-								info.t,
-								info.num,
-								info.lenSum,
-								info.lenMin,
-								info.lenAvg,
-								info.lenMax))
-						} else {
-							outfh.WriteString(fmt.Sprintf("%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d\t%.1f\t%.1f\t%.1f\t%d\t%d\t%.2f\t%.2f\t%.2f\n",
-								info.file,
-								info.format,
-								info.t,
-								info.num,
-								info.lenSum,
-								info.lenMin,
-								info.lenAvg,
-								info.lenMax,
+						fmt.Fprintf(outfh, "%s\t%s\t%s\t%d\t%d\t%d\t%.1f\t%d",
+							info.file,
+							info.format,
+							info.t,
+							info.num,
+							info.lenSum,
+							info.lenMin,
+							info.lenAvg,
+							info.lenMax)
+						if all {
+							fmt.Fprintf(outfh, "\t%.1f\t%.1f\t%.1f\t%d\t%d\t%d\t%.2f\t%.2f\t%.2f\t%.2f",
 								info.Q1,
 								info.Q2,
 								info.Q3,
 								info.gapSum,
 								info.N50,
+								info.L50,
 								info.q20,
 								info.q30,
-								info.gc))
+								info.avgQual,
+								info.gc)
 						}
+						if hasNX {
+							for _, x = range info.nx {
+								fmt.Fprintf(outfh, "\t%.0f", x)
+							}
+						}
+						outfh.WriteString("\n")
+						outfh.Flush()
 					}
 				}
 			}
@@ -276,6 +375,7 @@ Tips:
 
 		var wg sync.WaitGroup
 		token := make(chan int, config.Threads)
+		threadsFloat := float64(config.Threads) // just avoid repeated type conversion
 		var id uint64
 		for file := range chFile {
 			select {
@@ -288,9 +388,14 @@ Tips:
 			wg.Add(1)
 			id++
 			go func(file string, id uint64) {
+				startTime := time.Now()
 				defer func() {
 					wg.Done()
 					<-token
+
+					if !config.Quiet && len(files) > 1 {
+						chDuration <- time.Duration(float64(time.Since(startTime)) / threadsFloat)
+					}
 				}()
 
 				var gapSum uint64
@@ -298,8 +403,11 @@ Tips:
 
 				lensStats := util.NewLengthStats()
 
+				var errSum, avgQual float64
+				qual_map := seq.QUAL_MAP
 				var q20, q30 int64
 				var q byte
+				var qual int
 				var encodeOffset int = fqEncoding.Offset()
 				var seqFormat, t string
 				var record *fastx.Record
@@ -356,12 +464,15 @@ Tips:
 					if all {
 						if fastxReader.IsFastq {
 							for _, q = range record.Seq.Qual {
-								if int(q)-encodeOffset >= 20 {
+								qual = int(q) - encodeOffset
+								if qual >= 20 {
 									q20++
-									if int(q)-encodeOffset >= 30 {
+									if qual >= 30 {
 										q30++
 									}
 								}
+
+								errSum += qual_map[qual]
 							}
 						}
 
@@ -370,6 +481,8 @@ Tips:
 					}
 				}
 
+				fastxReader.Close()
+
 				if fastxReader.Alphabet() == seq.DNAredundant {
 					t = "DNA"
 				} else if fastxReader.Alphabet() == seq.RNAredundant {
@@ -387,6 +500,19 @@ Tips:
 					n50 = lensStats.N50()
 					l50 = lensStats.L50()
 					q1, q2, q3 = lensStats.Q1(), lensStats.Q2(), lensStats.Q3()
+
+					if errSum == 0 {
+						avgQual = 0
+					} else {
+						avgQual = -10 * math.Log10(errSum/float64(lensStats.Sum()))
+					}
+				}
+				var nx []float64
+				if hasNX {
+					nx = make([]float64, len(NX))
+					for i, x := range NX {
+						nx[i] = float64(lensStats.NX(x))
+					}
 				}
 
 				select {
@@ -405,7 +531,8 @@ Tips:
 						0, 0, 0, 0,
 						0, 0, 0, 0,
 						0, 0, 0,
-						0, 0, 0,
+						0, 0, 0, 0,
+						nx,
 						nil, id}
 				} else {
 					if basename {
@@ -416,10 +543,13 @@ Tips:
 					}
 					ch <- statInfo{file, seqFormat, t,
 						lensStats.Count(), lensStats.Sum(), gapSum, lensStats.Min(),
-						math.Round(lensStats.Mean(), 1), lensStats.Max(), n50, l50,
+						mathutil.Round(lensStats.Mean(), 1), lensStats.Max(), n50, l50,
 						q1, q2, q3,
-						math.Round(float64(q20)/float64(lensStats.Sum())*100, 2), math.Round(float64(q30)/float64(lensStats.Sum())*100, 2),
-						math.Round(float64(gcSum)/float64(lensStats.Sum())*100, 2),
+						mathutil.Round(float64(q20)/float64(lensStats.Sum())*100, 2),
+						mathutil.Round(float64(q30)/float64(lensStats.Sum())*100, 2),
+						mathutil.Round(avgQual, 2),
+						mathutil.Round(float64(gcSum)/float64(lensStats.Sum())*100, 2),
+						nx,
 						nil, id}
 				}
 			}(file, id)
@@ -427,9 +557,16 @@ Tips:
 
 		<-doneSendFile
 		wg.Wait()
+
 		close(ch)
 		<-done
 
+		if !config.Quiet && len(files) > 1 {
+			close(chDuration)
+			<-doneDuration
+			pbs.Wait()
+		}
+
 		select {
 		case <-cancel:
 			return
@@ -441,69 +578,72 @@ Tips:
 		}
 
 		// format output
-		columns := []prettytable.Column{
+		columns := []stable.Column{
 			{Header: "file"},
 			{Header: "format"},
 			{Header: "type"},
-			{Header: "num_seqs", AlignRight: true},
-			{Header: "sum_len", AlignRight: true},
-			{Header: "min_len", AlignRight: true},
-			{Header: "avg_len", AlignRight: true},
-			{Header: "max_len", AlignRight: true}}
+			{Header: "num_seqs", Align: stable.AlignRight, HumanizeNumbers: true},
+			{Header: "sum_len", Align: stable.AlignRight, HumanizeNumbers: true},
+			{Header: "min_len", Align: stable.AlignRight, HumanizeNumbers: true},
+			{Header: "avg_len", Align: stable.AlignRight, HumanizeNumbers: true},
+			{Header: "max_len", Align: stable.AlignRight, HumanizeNumbers: true},
+		}
 
 		if all {
-			columns = append(columns, []prettytable.Column{
-				{Header: "Q1", AlignRight: true},
-				{Header: "Q2", AlignRight: true},
-				{Header: "Q3", AlignRight: true},
-				{Header: "sum_gap", AlignRight: true},
-				{Header: "N50", AlignRight: true},
-				{Header: "Q20(%)", AlignRight: true},
-				{Header: "Q30(%)", AlignRight: true},
-				{Header: "GC(%)", AlignRight: true},
+			columns = append(columns, []stable.Column{
+				{Header: "Q1", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "Q2", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "Q3", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "sum_gap", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "N50", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "N50_num", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "Q20(%)", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "Q30(%)", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "AvgQual", Align: stable.AlignRight, HumanizeNumbers: true},
+				{Header: "GC(%)", Align: stable.AlignRight, HumanizeNumbers: true},
 				// {Header: "L50", AlignRight: true},
 			}...)
 		}
-
-		tbl, err := prettytable.NewTable(columns...)
-
-		checkError(err)
-		tbl.Separator = "  "
-
-		for _, info := range statInfos {
-			if !all {
-				tbl.AddRow(
-					info.file,
-					info.format,
-					info.t,
-					humanize.Comma(int64(info.num)),
-					humanize.Comma(int64(info.lenSum)),
-					humanize.Comma(int64(info.lenMin)),
-					humanize.Commaf(info.lenAvg),
-					humanize.Comma(int64(info.lenMax)))
-			} else {
-				tbl.AddRow(
-					info.file,
-					info.format,
-					info.t,
-					humanize.Comma(int64(info.num)),
-					humanize.Comma(int64(info.lenSum)),
-					humanize.Comma(int64(info.lenMin)),
-					humanize.Commaf(info.lenAvg),
-					humanize.Comma(int64(info.lenMax)),
-					humanize.Commaf(info.Q1),
-					humanize.Commaf(info.Q2),
-					humanize.Commaf(info.Q3),
-					humanize.Comma(int64(info.gapSum)),
-					humanize.Comma(int64(info.N50)),
-					humanize.Commaf(info.q20),
-					humanize.Commaf(info.q30),
-					humanize.Commaf(info.gc),
-					// humanize.Comma(info.L50),
-				)
+		if hasNX {
+			for _, x := range _NX {
+				columns = append(columns, stable.Column{Header: "N" + x, Align: stable.AlignRight, HumanizeNumbers: true})
 			}
 		}
-		outfh.Write(tbl.Bytes())
+
+		tbl := stable.New()
+		tbl.HeaderWithFormat(columns)
+		for _, info := range statInfos {
+			row := make([]interface{}, 0, len(columns))
+			row = append(row, info.file)
+			row = append(row, info.format)
+			row = append(row, info.t)
+			row = append(row, humanize.Comma(int64(info.num)))
+			row = append(row, info.lenSum)
+			row = append(row, info.lenMin)
+			row = append(row, info.lenAvg)
+			row = append(row, info.lenMax)
+
+			if all {
+				row = append(row, info.Q1)
+				row = append(row, info.Q2)
+				row = append(row, info.Q3)
+				row = append(row, info.gapSum)
+				row = append(row, info.N50)
+				row = append(row, info.L50)
+				row = append(row, info.q20)
+				row = append(row, info.q30)
+				row = append(row, info.avgQual)
+				row = append(row, info.gc)
+			}
+			if hasNX {
+				for _, x = range info.nx {
+					row = append(row, x)
+				}
+			}
+
+			tbl.AddRow(row)
+		}
+		outfh.Write(tbl.Render(style))
 	},
 }
 
@@ -526,11 +666,14 @@ type statInfo struct {
 	Q2 float64
 	Q3 float64
 
-	q20 float64
-	q30 float64
+	q20     float64
+	q30     float64
+	avgQual float64
 
 	gc float64
 
+	nx []float64
+
 	err error
 	id  uint64
 }
@@ -545,6 +688,9 @@ func init() {
 	statCmd.Flags().StringP("fq-encoding", "E", "sanger", `fastq quality encoding. available values: 'sanger', 'solexa', 'illumina-1.3+', 'illumina-1.5+', 'illumina-1.8+'.`)
 	statCmd.Flags().BoolP("basename", "b", false, "only output basename of files")
 	statCmd.Flags().StringP("stdin-label", "i", "-", `label for replacing default "-" for stdin`)
+	statCmd.Flags().StringSliceP("N", "N", []string{}, `append other N50-like stats as new columns. value range [0, 100], multiple values supported, e.g., -N 50,90 or -N 50 -N 90`)
+	statCmd.Flags().BoolP("skip-file-check", "S", false, `skip input file checking when given files or a file list.`)
+
 }
 
 func median(sorted []int64) int64 {
