@@ -522,7 +522,7 @@ rm corr.tsv seqkit.tsv joint.tsv
 fun(){
     rm -fr tests/bundler_test tests/bundler_merged.bam
     $app bam -N -1 $SPLICE_BAM -o tests/bundler_test
-    ($app bam -s $SPLICE_BAM 2>&1) | cut -f 1,8 | sed '1d' - > tests/bundler_stats_bulk.tsv
+    ($app bam -s $SPLICE_BAM 2>&1) | cut -f 1,8 | sed '1d' > tests/bundler_stats_bulk.tsv
     ($app bam -s tests/bundler_test/*.bam 2>&1) \
     | cut -f 1,8 | sed '1d' | $CSVTK -H -t summary -w 0 -g 1 -f "2:sum" > tests/bundler_stats_merged.tsv
 }
@@ -554,6 +554,15 @@ rm seqkit_fish.tsv
 #                       sana
 # ------------------------------------------------------------
 
+# sana/fastq test for IDs in separator
+fun(){
+	$app  sana tests/sana_sep_id.fq > tests/sana_output.fq
+}
+run sana_fastq_sep_id fun
+cmp tests/sana_output.fq <(tail -4 tests/sana_sep_id.fq)
+assert_equal $? 0
+rm -f tests/sana_output.fq
+
 # Regression test for sana/fasta
 fun(){
 	awk '{print ">" $1 "\n" $2}' tests/scat_test.tsv > tests/sana_test_input.fas
